package apps;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.ibm.rhapsody.apps.utils.IOServices;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPControlledFile;
import com.telelogic.rhapsody.core.IRPDependency;
import com.telelogic.rhapsody.core.IRPHyperLink;
import com.telelogic.rhapsody.core.IRPMatrixLayout;
import com.telelogic.rhapsody.core.IRPMatrixView;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPTableLayout;
import com.telelogic.rhapsody.core.IRPTableView;
import com.telelogic.rhapsody.core.IRPTag;

public class MBDGExpander {

	private final String OPEN_ELEMENT_PATTERN = "<element";
	private final String CLOSE_ELEMENT_PATTERN = "</element>";

	private final String TEMPLATE_COMMENT_COLLECTION_REPLICATED_START = "${MBDG_COLLECTION_REPLICATED_START}";
	private final String TEMPLATE_COMMENT_COLLECTION_REPLICATED_END = "${MBDG_COLLECTION_REPLICATED_END}";
	
	private static IRPApplication rhapsody;
	private StringBuilder buffer;
	private MBDGMapExpander mapExpander; 
	private static String ommroot;
	private static String tempDir = "";
	
	static void init(IRPApplication rhapsody) {
		setRhapsody(rhapsody);
		ommroot = rhapsody.getOMROOT();
	}

	public MBDGExpander(String s) {
		this.setBuffer(new StringBuilder(s));
		setMapExpander(new MBDGMapExpander(getBuffer()));
	}

	public MBDGExpander(StringBuilder template) {
		this.setBuffer(template);
		setMapExpander(new MBDGMapExpander(getBuffer()));
	}

	public MBDGExpander() {
		setBuffer(new StringBuilder());
		setMapExpander(new MBDGMapExpander(getBuffer()));
	}
	
	public void cleanBuffer() {
		buffer.delete(0, buffer.length());
		
	}	

	public void expand(MBDGElement element, int level) throws IOException {
		IRPModelElement docElement = element.getElement();
		IRPModelElement modelElement = element.getReferencedModelElement();

		expandByTags(docElement, modelElement);
		expandByPatterns(docElement, modelElement);
		expandByElementReferences(docElement);
		expandFigure(docElement, modelElement);
		expandCollection(docElement, modelElement, level);
		expandByIncludeFileOrHyperlink(element, modelElement);
		expandStyleName(docElement, level);
		//expandElementLevel(element, level);
		getMapExpander().expand();
	}		

	private void expandByTags(IRPModelElement docElement, IRPModelElement modelElement) {
		expandByTags(modelElement);
		expandByTags(docElement);		
	}

	public void expandByTags(IRPModelElement element) {
		if (element != null) {
			@SuppressWarnings("unchecked")
			List<IRPTag> tags = element.getAllTags().toList();
			for (IRPTag tag : tags) {
				String name = tag.getName();
				String value = tag.getValue();
				String pattern = String.format("${%s}", name);
				MBDGExpander expander = new MBDGExpander(value);
				String tagValue = expander.expand(element);
				expand(pattern, tagValue);				
			}
		}
	}	

	public void expandByPatterns(IRPModelElement docElement, IRPModelElement modelElement) throws IOException {
		IRPModelElement element = (modelElement != null ? modelElement : docElement);
		// First expand name as element's label 
		String label = element.getDisplayName();
		if (!label.equals("")) {
			getMapExpander().addPattern("${MBDG_NAME}", label);
		} else {
			// 	if label is empty expand name as element's name
			getMapExpander().addPattern("${MBDG_NAME}", element.getName());
		}
		getMapExpander().addPattern("${MBDG_DESCRIPTION}", element.getDescription());
		getMapExpander().addPattern("${MBDG_PROJECT_NAME}", element.getProject().getName());
	}

	public void expandFigure(IRPModelElement docElement, IRPModelElement modelElement) throws IOException {
		if (modelElement != null) {
			expandQuery(getBuffer(), modelElement.getOwner());
			expand("${MBDG_FIGURE_NAME}", modelElement.getName());
			expand("${MBDG_FIGURE_ID}", modelElement.getGUID());
			expand("(ObjectModelDiagram)", "("+getMatatype(modelElement)+")");
		}
	}

	public String getMatatype(IRPModelElement e) {
		return e.getMetaClass().replace(" ", "_");
	}

	public void expandByElementReferences(IRPModelElement docElement) throws IOException {	
		@SuppressWarnings("unchecked")
		List<IRPDependency> dependencies = docElement.getDependencies().toList();
		for (IRPDependency dependency : dependencies) {
			IRPModelElement dependsOn = dependency.getDependsOn();
			if (dependsOn != null) {
				if (MBDGUtils.isType(dependency, "DocDescriptionReference")) {
					getMapExpander().addPattern("${MBDG_DESCRIPTION}", dependsOn.getDescription());
				}			
			}
		}
	}	

	public void expandByIncludeFileOrHyperlink(MBDGElement docElement, IRPModelElement modelElement) throws IOException {	
		if (MBDGUtils.isType(docElement, "DocIncludeFile") || MBDGUtils.isType(docElement, "DocHyperlink")) {
			if (modelElement instanceof IRPControlledFile) {
				IRPControlledFile controlledFile = (IRPControlledFile) modelElement;
				getMapExpander().addPattern("${MBDG_INCLUDEFILE}", controlledFile.getFullPathFileName());
			} else if (modelElement instanceof IRPHyperLink) {
				IRPHyperLink hyperLink = (IRPHyperLink) modelElement;
				getMapExpander().addPattern("${MBDG_INCLUDEFILE}", hyperLink.getURL());
			} else if (!docElement.isRoot() && MBDGUtils.isDanglingElement(docElement.getElement())) {
				cleanBuffer();
			}
			expandIncludingType(docElement.getElement());	
		}
	}

	private void expandIncludingType(IRPModelElement docElement) {
		String includingType = MBDGUtils.getTagValue(docElement, "includingType");
		if (includingType == null || includingType.equals("")) {
			includingType = "physical embedding";
		}
		includingType = includingType.replace("_", " ");
		expand("physical embedding", includingType);
		expand("dynamic link", includingType);
	}	

	public void expandCollection(IRPModelElement docElement, IRPModelElement scopeElement, int level) throws IOException {
		String keyword = MBDGUtils.getTagValue(docElement, "Keyword");
		if (keyword == null || keyword.equals("")) {
			keyword = "${MBDG_CONTENT}";
		}
		if (keyword != null && getMapExpander().containsKey(keyword)) {
			StringBuilder collectionItemContent = extractCollectionItem(docElement);
			if (collectionItemContent != null && collectionItemContent.length() > 0) {
				// Specify list style according to tag in list
				expandListNumberingStyle(docElement);
				@SuppressWarnings("unchecked")
				List<IRPDependency> dependencies = docElement.getDependencies().toList();
				for (IRPDependency dependency : dependencies) {
					IRPModelElement collectionItem = dependency.getDependsOn();
					if (collectionItem != null) {
						if (MBDGUtils.isType(dependency, "DocElementReference")) {
							if (collectionItem instanceof IRPTableLayout) {
								if (MBDGUtils.isType(docElement, "DocList")) {
									expandListItems(docElement, collectionItem, scopeElement, keyword, collectionItemContent);
								}								
							} else if (collectionItem instanceof IRPTableView) {
								IRPTableView table = (IRPTableView) collectionItem;
								if (MBDGUtils.isType(docElement, "DocList")) {
									expandListItems(docElement, table, scopeElement, keyword, collectionItemContent);
								} if (MBDGUtils.isType(docElement, "DocTable")) {
									expandTableItems(docElement, table, scopeElement, keyword, collectionItemContent);
								}								
							} else if (collectionItem instanceof IRPMatrixView) {
								IRPMatrixView matrix = (IRPMatrixView) collectionItem;
								expandMatrixItems(docElement, matrix, keyword, collectionItemContent);						
							} else {
								expandListItem(docElement, collectionItem, keyword, collectionItemContent);
							}
						}
					}
				}
			}
		}	
	}

	private void expandListNumberingStyle(IRPModelElement docElement) {
		if (MBDGUtils.isType(docElement, "DocList")) {
			String numberingStyle = MBDGUtils.getTagValue(docElement, "numberingStyle");
			if (numberingStyle == null || numberingStyle.equals("")) {
				numberingStyle = "arabic";
			}
			numberingStyle = numberingStyle.replace("_", " ");
			expand("arabic", numberingStyle);
		}
	}
	
	private void expandMatrixItems(IRPModelElement docElement, IRPMatrixView matrix, String keyword, StringBuilder collectionItemContent) {
		IRPCollection pCommandInitialization = rhapsody.createNewCollection();
		pCommandInitialization.addItem(matrix);
		IRPCollection pCommandResult = rhapsody.createNewCollection();
		rhapsody.executeCommand("GetResultMatrixFromTable", pCommandInitialization, pCommandResult);
		
		expandMatrixScopeAndElementTypes(matrix);
		
		@SuppressWarnings("unchecked")
		List<String> datas = pCommandResult.toList();
		int rowsCount = Integer.parseInt(datas.get(0));
		int colsCount = Integer.parseInt(datas.get(1));
		datas.remove(0);
		datas.remove(0);
		if (rowsCount > 0) {
			for (int cols = 0; cols < colsCount; cols++) {
				String header = datas.get(0);
				datas.remove(0);
				String pattern1 = String.format("MBDG_EXIST_HEADER%d", cols);
				String pattern2 = String.format("MBDG_TABLE_HEADER%d", cols);
				
				getMapExpander().addPattern(pattern1, "true");				
				getMapExpander().addPattern(pattern2, header);
			}					
			expandTableItem(docElement, matrix, keyword, collectionItemContent, --rowsCount, colsCount, datas);
		}
	}

	private void expandMatrixScopeAndElementTypes(IRPMatrixView matrix) {
		String fromScope = getScopeName(matrix.getFromScope());
		String toScope = getScopeName(matrix.getToScope());

		String fromElementTypes = "";
		String toElementTypes = "";
		IRPMatrixLayout matrixLayout = matrix.getItsMatrixLayout();
		if (matrixLayout != null) {
			fromElementTypes  = MBDGUtils.toString(matrixLayout.getFromElementTypes());
			toElementTypes  = MBDGUtils.toString(matrixLayout.getToElementTypes());
		}
		
		getMapExpander().addPattern("MBDG_MATRIX_FROMSCOPE", fromScope);	
		getMapExpander().addPattern("MBDG_MATRIX_FROMSELEMENTTYPES", fromElementTypes);
		getMapExpander().addPattern("MBDG_MATRIX_TOSCOPE", toScope);
		getMapExpander().addPattern("MBDG_MATRIX_TOELEMENTTYPES", toElementTypes);
	}

	private void expandTableItems(IRPModelElement docElement, IRPTableView table, IRPModelElement scopeElement, String keyword, StringBuilder collectionItemContent) {
		IRPCollection pCommandInitialization = rhapsody.createNewCollection();
		pCommandInitialization.addItem(table);
		IRPCollection pCommandResult = rhapsody.createNewCollection();
		rhapsody.executeCommand("GetResultMatrixFromTable", pCommandInitialization, pCommandResult);
		IRPModelElement scope = scopeElement;
		if (scope == null) {
			scope = getScope(table.getScope());
		}
		expandQuery(getBuffer(), scope);
		@SuppressWarnings("unchecked")
		List<String> datas = pCommandResult.toList();
		int rowsCount = Integer.parseInt(datas.get(0));
		int colsCount = Integer.parseInt(datas.get(1));
		datas.remove(0);
		datas.remove(0);
		if (rowsCount > 0) {
			for (int cols = 0; cols < colsCount; cols++) {
				String header = datas.get(0);
				datas.remove(0);
				String pattern1 = String.format("MBDG_EXIST_HEADER%d", cols);
				String pattern2 = String.format("MBDG_TABLE_HEADER%d", cols);
				
				getMapExpander().addPattern(pattern1, "true");				
				getMapExpander().addPattern(pattern2, header);
			}					
			expandTableItem(docElement, table, keyword, collectionItemContent, --rowsCount, colsCount, datas);
		}
	}

	private IRPModelElement getScope(IRPCollection col) {
		IRPModelElement scope;
		@SuppressWarnings("unchecked")
		List<IRPModelElement> scopeList = col.toList();
		if (scopeList.size() > 0) {
			scope = scopeList.get(0);
		} else {
			scope = rhapsody.activeProject();
		}
		return scope;
	}	
	
	private String getScopeName(IRPCollection col) {
		String res = "";
		IRPModelElement scope = getScope(col);	
		if (scope != null) {
			res = scope.getName();
		}
		return res;
	}

	private void expandTableItem(IRPModelElement docElement, IRPModelElement element, String keyword, StringBuilder collectionItemContent, int rowsCount, int colsCount, List<String> datas) {
		for (int rows = 0; rows < rowsCount; rows++) {
			StringBuilder content = new StringBuilder(collectionItemContent);
			MBDGTemplateHandlesIdsUpdater handlesUpdater = new MBDGTemplateHandlesIdsUpdater(content);
			handlesUpdater.update();

			MBDGExpander expander = new MBDGExpander(content);
			for (int cols = 0; cols < colsCount; cols++) {
				String data = datas.get(rows*colsCount+cols);
				
				String pattern1 = String.format("MBDG_EXIST_CELL_VALUE%d", cols);
				String pattern2 = String.format("MBDG_CELL_VALUE%d", cols);
				
				expander.getMapExpander().addPattern(pattern1, "true");
				expander.getMapExpander().addPattern(pattern2, data);
				
				expander.getMapExpander().expand();
			}	
			getMapExpander().addTemplatePattern(keyword, content);	
		}		
	}

	private void expandListItems(IRPModelElement docElement, IRPModelElement modelElement, IRPModelElement scopeElement, String keyword, StringBuilder collectionItemContent) {
		List<Object> objects = MBDGUtils.getResultList(modelElement, scopeElement);
		for (Object object : objects) {
			expandListItem(docElement, (IRPModelElement)object, keyword, collectionItemContent);
		}
	}

	protected void expandListItem(IRPModelElement docElement, IRPModelElement element, String keyword, StringBuilder collectionItemContent) {
		StringBuilder content = new StringBuilder(collectionItemContent);
		MBDGTemplateHandlesIdsUpdater handlesUpdater = new MBDGTemplateHandlesIdsUpdater(content);
		handlesUpdater.update();
		MBDGExpander expander = new MBDGExpander(content);
		expander.expand("${MBDG_COLLECTION_ITEM_NAME}", element.getName());
		expander.expand("${MBDG_COLLECTION_ELEMENT_ID}", element.getGUID());	
		expander.expandTemplateComment("${MBDG_CONTENT}", "");
		expandQuery(content, element.getOwner());
		getMapExpander().addTemplatePattern(keyword, content);
	}
	
	protected void expandQuery(StringBuilder content, IRPModelElement e) {
		MBDGExpander expander = new MBDGExpander(content);
		String templateQuery = getFirstTemplateQuery(content);
		if (!templateQuery.equals("")) {
			String query = getElementQuery(e);
			if (!query.equals("")) {
				expander.expand(templateQuery, query+"/NestedElements");
			}
		}
	}

	protected String getFirstTemplateQuery(StringBuilder content) {
		String query = "";
		String queryOpenTag = "<query>";
		String queryCloseTag = "</query>";
		int open = content.indexOf(queryOpenTag, 0);
		if (open >= 0) {
			int close = content.indexOf(queryCloseTag, open);
			if (close >= 0) {
				query = content.substring(open+queryOpenTag.length(), close);
			}
		}
		return query;
	}
	
	protected String getFirstBlockByTag(String tag, StringBuilder content) {
		String block = "";
		List<String> blocks = getBlocksByTag(tag, true, content);
		if (blocks.size() > 0) {
			block = blocks.get(0);
		}
		return block;
	}
	
	protected List<String> getAllBlocksByTag(String tag, StringBuilder content) {
		return getBlocksByTag(tag, false, content);
	}	
	
	protected List<String> getBlocksByTag(String tag, boolean onlyFirst, StringBuilder content) {
		List<String> blocks = new ArrayList<String>();
		String openTag = tag;
		String closeTag = (tag.endsWith(">") ? tag : (tag+">")).replace("<", "</");
		int open = content.indexOf(openTag, 0);
		while (open >= 0) {
			int close = content.indexOf(closeTag, open);
			if (close > open) {
				String block = content.substring(open+openTag.length(), close);
				blocks.add(block);
				if (onlyFirst) {
					break;
				}
				open = content.indexOf(openTag, close);
			} else {
				open = -1;
			}
		}		
		return blocks;
	}	
	
	public String getRhapsodySchemaPath(String dir, String content) {
		String path = "";
		String schemasBlock = getFirstBlockByTag("<schemas>", new StringBuilder(content));
		List<String> blocks = getAllBlocksByTag("<schema>", new StringBuilder(schemasBlock));
		for (String block : blocks) {
			if (block.contains("RhapsodySchema_schema")) {
				List<String> propertiesBlocks = getAllBlocksByTag("<property", new StringBuilder(block));
				for (String propertiesBlock : propertiesBlocks) {
					if (propertiesBlock.contains("schemas") && propertiesBlock.contains(".xsd")) {
						int start = propertiesBlock.indexOf("schemas");
						int end = propertiesBlock.lastIndexOf(".xsd");
						String path1 = propertiesBlock.substring(start, end);
						path = dir+File.separator+path1+".xsd";
						break;
					}
				}
			}
		}
		return path;
	}	

	protected String getElementQuery(IRPModelElement e) {
		StringBuilder query = new StringBuilder();
		while (e != null) {
			String name;
			if (e instanceof IRPProject) {
				name = e.getMetaClass();
			} else {
				name = e.getUserDefinedMetaClass();
			}
			String pluralName = name+"s";
			IRPStereotype newTerm = e.getNewTermStereotype();
			if (newTerm != null) {
				String pluralName1 = newTerm.getPropertyValue("Model.Stereotype.PluralName");
				if (!pluralName1.equals("")) {
					pluralName = pluralName1;
				}
			} else {
				if (pluralName.endsWith("ss")) {
					pluralName = name.substring(0, name.length()-1)+"ses";
				} else if (pluralName.endsWith("y")) {
					pluralName = name.substring(0, name.length()-1)+"ies";
				} else if (pluralName.equals("Stereotypes")) {
					pluralName = "ContainedStereotypes";
				}
			}
			if (query.length() != 0) {
				query.insert(0, "/");
			}
			query.insert(0,pluralName+"/"+name);
			e = e.getOwner();
		}
		String s = query.toString();
		s = s.replace(" ", "_");
		return s;
	}

	protected StringBuilder extractCollectionItem(IRPModelElement docElement) {
		final String openTag = "<element";
		final String closeTag = "</element>";

		String content = "";

		int start = getBuffer().indexOf(TEMPLATE_COMMENT_COLLECTION_REPLICATED_START);
		int end = getBuffer().indexOf(TEMPLATE_COMMENT_COLLECTION_REPLICATED_END);
		if (start >= 0 && end >= 0) {
			start = getBuffer().indexOf(openTag, start);

			String str = getBuffer().substring(start, end);
			end = str.lastIndexOf(closeTag);
			if (end >= 0) {
				content = str.substring(0, end+closeTag.length());			

				end = start+content.length();
				getBuffer().replace(start, end, "");  


			}
		}
		return new StringBuilder(content); 
	}

	protected String replaceTemplateComments() {
		expandTemplateComment(TEMPLATE_COMMENT_COLLECTION_REPLICATED_START, "");
		return TEMPLATE_COMMENT_COLLECTION_REPLICATED_END;
	}	

	public void expandStyleName(IRPModelElement docElement, int level) {
		final String tag1 = "tag=\"paragraph\"";
		final String tag2 = "<property name=\"style name\" value=\"1\"></property>";
		String pattern = "<property name=\"style name\" value=\"${Numbering}\"></property>";

		int start = getBuffer().indexOf(tag1);
		start = getBuffer().indexOf(tag2, start);

		if (start >= 0) {
			String styleName = MBDGUtils.getTagValue(docElement, "HeadingStyle");
			if (styleName == null) {
				styleName = "${AUTO}";
			}
			if (styleName.equals("${AUTO}")) {
				styleName = Integer.toString(level);
			}
			if (MBDGUtils.isType(docElement, "DocIncludeFile")) {
				styleName = "0";
			}			
			String pattern1 = pattern.replace("${Numbering}", styleName);
			expand(tag2, pattern1);
		}
	}	
	
	public void expandElementLevel(MBDGElement docElement, int level) {
		String value = String.format("_element_level*1+%d-1", level);
		expand("_element_level*1", value);
	}	

	public void expandTemplateComment(String pattern, String patternContent) {
		String templateCommentTag = "tag=\"template comment\"";
		int start = getBuffer().indexOf(pattern);
		if (start >= 0) {
			String comment = getBuffer().substring(0, start);
			int open = comment.lastIndexOf(OPEN_ELEMENT_PATTERN);
			int med = comment.lastIndexOf(templateCommentTag);
			int close = getBuffer().indexOf(CLOSE_ELEMENT_PATTERN, start);
			if (open >= 0 && close >= 0 && open < med && close > med) {
				getBuffer().replace(open, close+CLOSE_ELEMENT_PATTERN.length(), patternContent) ; 
			}
		}
	}
	
	public boolean expand(String pattern, StringBuilder value) {
		return expand(pattern, value.toString());
	}

	public boolean expand(String pattern, String value) {
		boolean res = false;
		if (!pattern.equals(value)) {
			int start = 0;
			while(start >= 0) {
				start = getBuffer().indexOf(pattern, start);
				if (start >= 0) {
					int end = start + pattern.length();
					getBuffer().replace(start, end, value) ;
					res = true;
					start++;
				}
			}
		}
		return res;
	}
	
	public boolean expandFirst(String pattern, String value) {
		boolean res = false;
		if (!pattern.equals(value)) {
			int start = getBuffer().indexOf(pattern);
			if (start >= 0) {
				int end = start + pattern.length();
				getBuffer().replace(start, end, value) ;
				res = true;
			}
		}
		return res;
	}	

	public String expand(String pattern) {
		if (pattern == null) {
			return null;
		}
		setBuffer(new StringBuilder(pattern));  
		expand("${OMROOT}", ommroot);
		expand("${TEMP_DIR}", tempDir);
		return getBuffer().toString();
	}

	public String expandPath(String pattern) {
		String path = expand(pattern);
		path = path.replace("\\", "/");
		return path;
	}

	public String expand(IRPModelElement e) {
		String name = e.getDisplayName();
		if (name.equals("")) {
			name = e.getName();
		}
		expand("${NAME}", name);
		return getBuffer().toString();
	}
	public String collapsePath(String path) {
		if (path != null) {
			setBuffer(new StringBuilder(path));
			expand(ommroot, "${OMROOT}");
		}
		return getBuffer().toString();
	}	

	public void expandFile(String path, String pattern, String value) throws IOException {
		File file = new File(path);
		String contents = IOServices.getContents(file);
		setBuffer(new StringBuilder(contents));
		expand(pattern, value);
		IOServices.setContents(file, getBuffer().toString());
	}		

	public static IRPApplication getRhapsody() {
		return rhapsody;
	}

	public static void setRhapsody(IRPApplication rhapsody) {
		MBDGExpander.rhapsody = rhapsody;
	}

	public StringBuilder getBuffer() {
		return buffer;
	}

	public void setBuffer(StringBuilder buffer) {
		this.buffer = buffer;
	}

	public static String getTempDir() {
		return tempDir;
	}

	public static void setTempDir(String tempDir) {
		MBDGExpander.tempDir = tempDir;
	}

	public MBDGMapExpander getMapExpander() {
		return mapExpander;
	}

	public void setMapExpander(MBDGMapExpander mapExpander) {
		this.mapExpander = mapExpander;
	}
}
