package apps;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.ibm.rhapsody.apps.utils.IOServices;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPModelElement;

public class MBDGTemplateGenerator {
	private MBDGConfiguration configuration;
	private IRPApplication rhapsody;

	public MBDGTemplateGenerator(IRPApplication rhapsody, MBDGConfiguration configuration) throws IOException {
		this.rhapsody = rhapsody;
		this.configuration = configuration;
	}	
	
	
	public void unzipTemplates(IRPModelElement element) throws IOException {
		Set<String> templatePaths = new HashSet<String>();
		
		getPredefinedTemplates(templatePaths);
		getUserDefinedTemplates(element, templatePaths);

		for (String templatePath : templatePaths) {
			if (shouldUnzipTemplate(templatePath)) {
				unzipTemplate(templatePath);
			}
		}
	}


	protected void getUserDefinedTemplates(IRPModelElement element, Set<String> templatePaths) {
		String path = MBDGUtils.getTagValue(element, "templateFile");
		templatePaths.add(path);
		@SuppressWarnings("unchecked")
		List<IRPModelElement> nestedElements = element.getNestedElements().toList();
		for (IRPModelElement nestedElement : nestedElements) {
			if (nestedElement instanceof IRPInstance) {
				IRPInstance part = (IRPInstance) nestedElement;	
				getUserDefinedTemplates(part.getOtherClass(), templatePaths);
			}
		}
	}


	protected void getPredefinedTemplates(final Set<String> templatePaths) {
		new File(configuration.getTemplatesDirectory()).list(new FilenameFilter() {
			public boolean accept(File dir, String name) {
				if (name.endsWith(".dta")) {
					templatePaths.add(dir+File.separator+name);
				}
				return true;
			}
		});
	}	
	
	protected boolean shouldUnzipTemplate(String templatePath) {
		File templateFile = new File(templatePath);
		File templateXmlFile = configuration.getTemplateXmlFile(templateFile);
		boolean res = (!templateXmlFile.exists() || (templateXmlFile.lastModified() < templateFile.lastModified()));
		return res;
	}
	
	public String unzipTemplate(String templatePath) throws IOException {
		String dir = configuration.getTemplateUnzippedDirectory(templatePath);
		if (IOServices.doesExist(dir)) {
			IOServices.deleteDir(dir);
		}
		MBDGZipUtility.unzip(templatePath, dir);
		return dir;
	}
	
	public String getTemplatePathFromTemplateDir(String dir) {
		return dir+File.separator+"template.xml";
	}	
	
	public void cleanWorkarea() throws IOException {
    	try {
			if (IOServices.doesExist(configuration.getGeneratedTemplatePath())) {
	    		IOServices.deleteFile(configuration.getGeneratedTemplatePath());
	    	}
	    	if (IOServices.doesExist(configuration.getDocumentSpecificationPath())) {
	    		IOServices.deleteFile(configuration.getDocumentSpecificationPath());
	    	}  
	    	if (IOServices.doesExist(configuration.getWorkareaDirectory())) {
	    		IOServices.deleteDir(configuration.getWorkareaDirectory());
	    	}
			IOServices.createDir(configuration.getWorkareaDirectory());
    	} catch (IOException e) {
    		throw new IOException(String.format("Failed to initialize correctly workarea [%s]. Possible cause: locked or readonly files.", configuration.getWorkareaDirectory()));
    	}
	}
	
	public void zipTemplate(StringBuilder template) throws IOException {
		IOServices.setContents(configuration.getGeneratedTemplateXmlFile(), template.toString());
    	MBDGZipUtility.zipDirectory(configuration.getGeneratedTemplateDirectory(), configuration.getGeneratedTemplatePath());
	}


	public void createTemplate(MBDGElement root) throws IOException {
		String templatePath = MBDGUtils.getRootTemplatePath(root);
		String templateUnzippedDir = configuration.getTemplateUnzippedDirectory(templatePath);
		IOServices.copyDirRecursive(templateUnzippedDir, configuration.getGeneratedTemplateDirectory(), true);
	}	
	
	public void updatePortInFiles(int port) throws IOException {
		String pattern1 = "http://localhost:27463/Rational/Rhapsody/";
		String pattern2 = "http://localhost:27464/Rational/Rhapsody/";
		String value = String.format("http://localhost:%d/Rational/Rhapsody/", port);
		File[] files = collectFiles();
		for (File file : files) {
			String path = file.getAbsolutePath();
			new MBDGExpander().expandFile(path, pattern1, pattern2);
			new MBDGExpander().expandFile(path, pattern2, value);
		}
	}

	protected File[] collectFiles() {
		File dir = new File(configuration.getWorkareaDirectory());
		List<File> files = new ArrayList<File>();
		collectFiles(dir, files);
		return files.toArray(new File[files.size()]);
	}

	protected void collectFiles(final File dir, final List<File> files) {
		dir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				if (pathname.isDirectory()) {
					collectFiles(pathname, files);
				} else {
					String name = pathname.getName();
					if (name.equals("__RPE_MAPPINGS__") || name.equals("generated.dsx")) {
						files.add(pathname);
					}
				}
				return true;
			}
		});
	}	
	
	protected void replacePatternInFile(String path, String pattern, String value) throws IOException {
		String contents = IOServices.getContents(path);
		MBDGExpander expander = new MBDGExpander(contents);
		expander.expand(pattern, value);
		IOServices.setContents(new File(path), expander.getBuffer().toString());
	}	
	

	public void updateSchema() throws IOException {
		String existingSchemaPath = getSchemaPath();
		String rhapsodySchemaPath = getRhapsodySchemaPath();
		updateSchema(rhapsodySchemaPath, existingSchemaPath);
	}
	
	protected void exportSchema() {
		String fileSchema = getRhapsodySchemaPath();
		IRPCollection params = rhapsody.createNewCollection();
		params.setSize(3);
		// schemaFileName
		params.setString( 1, fileSchema);
		// baseURL
		params.setString( 2, "rhp");
		// bNewTermSupportInFull
		params.setString( 3, "true");
		rhapsody.executeCommand( "GetSchema", params, null);
		params.empty();
	}
	
	protected String getRhapsodySchemaPath() {
		File fileSchema = new File(configuration.getDocumentSpecificationsDirectory()+File.separator+"RhapsodySchema.xsd");
		return fileSchema.getAbsolutePath();
	}	
	
	protected String getSchemaPath() throws IOException {
		String path = "";
		String contents = IOServices.getContents(configuration.getGeneratedTemplateXmlFile());
		MBDGExpander expander = new MBDGExpander();
		path = expander.getRhapsodySchemaPath(configuration.getGeneratedTemplateDirectory(), contents);
		return path;
	}
	
	protected void updateSchema(String sourcePath, String targetPath) throws IOException {
		try {
			String contents = IOServices.getContents(sourcePath);
			IOServices.setContents(new File(targetPath), contents);
		} catch (IOException e) {
			IOException e1 = new IOException(String.format("Failed to update Rhapsody default Schema=[%s] with current Schema=[%s]",targetPath, sourcePath));
			throw e1;
		}
	}	
}
