package apps;

import java.io.File;
import java.io.IOException;
import java.util.List;

import com.ibm.rhapsody.apps.utils.IOServices;

public class MBDGTemplateProvider {

	private MBDGSimpleModel simpleModel;
	private MBDGConfiguration configuration;

	public MBDGTemplateProvider(MBDGSimpleModel simpleModel, MBDGConfiguration configuration) {
		this.simpleModel = simpleModel;
		this.configuration = configuration;
	}

	public void assignTemplates() throws IOException {
		MBDGElement root = simpleModel.getRoot();
		assignRootTemplate(root);
		provide(root);
	}

	protected void provide(MBDGElement element) throws IOException {
		List<MBDGElement> descendants = element.getDescendants();
		for (MBDGElement descendant : descendants) {
			assignTemplate(descendant);
			provide(descendant);
		}	
	}

	protected void assignRootTemplate(MBDGElement root) throws IOException {
		StringBuilder template = loadRootTemplate(root);
		updateTemplate(root, template);		
	}
	
	protected void assignTemplate(MBDGElement element) throws IOException {
		StringBuilder template = loadContent(element);
		updateTemplate(element, template);	
	}
	
	protected void updateTemplate(MBDGElement element, StringBuilder template) throws IOException {
		MBDGTemplateHandlesIdsUpdater handlesUpdater = new MBDGTemplateHandlesIdsUpdater(template);
		handlesUpdater.update();	
		MBDGContextExpander contextExpander = new MBDGContextExpander(template);
		contextExpander.expandReplaceableQueries(element);		
		contextExpander.collectContextHandleAndQuery(element);
		contextExpander.removeReplaceableContainerAndUpdateContext(element);
		element.setTemplate(template);
	}	

	private StringBuilder loadContent(MBDGElement element) throws IOException {
		StringBuilder template;
		boolean isDocument = MBDGUtils.isType(element, "DocDocument");
		if (isDocument == true) {
			template = extractEmptyTemplateContent();
		} else {
			template = extractTemplateContent(element);
		}
		return template;
	}	
	
	protected StringBuilder extractTemplateContent(StringBuilder template, MBDGElement element, int level) throws IOException {
		StringBuilder content = extractContent(template);
		MBDGExpander expander = new MBDGExpander(content);
		expander.expand(element, level);
		return expander.getBuffer();
	}	
	
	protected StringBuilder extractTemplateContent(MBDGElement docElement, int level) throws IOException {
		StringBuilder template = loadTemplate(docElement);
		StringBuilder content = extractTemplateContent(template, docElement, level);
    	return content;
	}
	
	protected StringBuilder loadContentTemplate() throws IOException {
		// Load root template which will be a root container for other templates
		String path = new MBDGExpander().expand(MBDGConfiguration.ContentDocumentTemplatePath);
		StringBuilder template = loadTemplate(path);
		return template;
	}
	
	protected StringBuilder loadRootTemplate(MBDGElement root) throws IOException {
		StringBuilder template;
		boolean isDocument = MBDGUtils.isType(root, "DocDocument");
		if (isDocument == true) {
			template = loadTemplate(root);
		} else {
			// Load root template which will be a root container for other templates
			String path = new MBDGExpander().expand(MBDGConfiguration.DocumentTemplatePath);
			template = loadTemplate(path);
		}		
		return template;
	}	
	
	protected StringBuilder loadTemplate(String templatePath) throws IOException {
		String patternPath = configuration.getTemplateXmlPath(templatePath);
    	String template = IOServices.getContents(new File(patternPath));
    	StringBuilder templateSB = new StringBuilder(template);
		MBDGTemplateHandlesIdsUpdater handlesUpdater = new MBDGTemplateHandlesIdsUpdater(templateSB);
		handlesUpdater.update(); 	
		return templateSB;
	}	
	
	protected StringBuilder extractEmptyTemplateContent() throws IOException {
		String path = new MBDGExpander().expand(MBDGConfiguration.ContentDocumentTemplatePath);
		StringBuilder template = loadTemplate(path);		
		StringBuilder content = extractContent(template);
		return content;
	}	
	
	protected StringBuilder extractTemplateContent(MBDGElement element) throws IOException {
		StringBuilder template = loadTemplate(element);
		StringBuilder content = extractContent(template);
		return content;
	}
	
	protected StringBuilder extractContent(StringBuilder template) {
		final String openTag = "<content>";
		final String closeTag = "</content>";
		String content = "";
		int start = template.indexOf(openTag);
		int end = template.lastIndexOf(closeTag);
		if (start >= 0 && end >= 0) {
			content = template.substring(start+openTag.length(), end);
		}		
		return new StringBuilder(content);
	}
	
	protected StringBuilder loadTemplate(MBDGElement docElement) throws IOException {
		// Load root template which will be a root container for other templates
		String path = MBDGUtils.getTemplatePathByTag(docElement);
		StringBuilder template = loadTemplate(path);
		return template;
	}	

	protected StringBuilder loadContentTemplate(MBDGElement element) throws IOException {	
		StringBuilder template;
		boolean isDocument = MBDGUtils.isType(element, "DocDocument");
		if (isDocument == true) {
			template = loadContentTemplate();
		} else {
			template = loadTemplate(element);
		}
		return template;
	}		
}
